% demo_LSDD.m
%
% (c) Masashi Sugiyama, Department of Compter Science, Tokyo Institute of Technology, Japan.
%     sugi@cs.titech.ac.jp,     http://sugiyama-www.cs.titech.ac.jp/~sugi/software/LSDD/

clear all

rand('state',0);
randn('state',0);

%%%%%%%%%%%%%%%%%%%%%%%%% Generating training data

dataset=1;
disp(sprintf('dataset=%g',dataset))

d=1;
switch dataset
  case 1 % same distribution
mu2=0;
  case 2
mu2=1;
end
n1=200;
n2=200;
mu1=0;
sigma1=1;
sigma2=1;
x1=repmat([mu1;zeros(d-1,1)],[1 n1])+sigma1*randn(d,n1);
x2=repmat([mu2;zeros(d-1,1)],[1 n2])+sigma2*randn(d,n2);
L2dist_true=(1/sqrt(pi))*(1-exp(-(1/4)*(mu1-mu2)^2));

tmp=std([x1 x2],[],2);
x1=x1./repmat(tmp,[1 n1]);
x2=x2./repmat(tmp,[1 n2]);

xmin=min([x1 x2]);
xmax=max([x1 x2]);
xmargin=(xmax-xmin)*0.1;
xdisp=linspace(xmin-xmargin,xmax+xmargin,100);
p1_xdisp=1/sqrt(2*pi*sigma1^2)*exp(-(xdisp-mu1).^2/(2*sigma1^2));
p2_xdisp=1/sqrt(2*pi*sigma2^2)*exp(-(xdisp-mu2).^2/(2*sigma2^2));
f_xdisp=p1_xdisp-p2_xdisp;

figure(1); clf; 
subplot(2,1,1);hold on
set(gca,'FontName','Helvetica')
set(gca,'FontSize',12)
title('Data samples');
ymax=max(hist(x1,linspace(min(xdisp),max(xdisp),20)));
hist(x1,linspace(min(xdisp),max(xdisp),20))
h = findobj(gca,'Type','patch'); set(h,'FaceColor','y')
legend('x_i')
xlabel('x');
tmp=axis; axis([min(xdisp) max(xdisp) 0 ymax*1.1])
subplot(2,1,2);hold on
set(gca,'FontName','Helvetica')
set(gca,'FontSize',12)
ymax=max(hist(x2,linspace(min(xdisp),max(xdisp),20)));
hist(x2,linspace(min(xdisp),max(xdisp),20))
h = findobj(gca,'Type','patch'); set(h,'FaceColor','c')
legend('x''_{i''}')
xlabel('x');
tmp=axis; axis([min(xdisp) max(xdisp) 0 ymax*1.1])
set(gcf,'PaperUnits','centimeters');
set(gcf,'PaperPosition',[0 0 12 9]);
%print('-depsc',sprintf('samples%g',dataset))
print('-dpng',sprintf('samples%g',dataset))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[L2dist_LSDD,fh_xdisp]=LSDD(x1,x2,xdisp);

ymin=min([p1_xdisp p2_xdisp f_xdisp fh_xdisp]);
ymax=max([p1_xdisp p2_xdisp f_xdisp fh_xdisp]);
ymargin=(ymax-ymin)*0.1;

figure(2); clf; hold on
set(gca,'FontName','Helvetica')
set(gca,'FontSize',12)
plot(xdisp,p1_xdisp,'g-.','LineWidth',2)
plot(xdisp,p2_xdisp,'b:','LineWidth',5)
plot(xdisp,f_xdisp,'k--','LineWidth',2)
plot(xdisp,fh_xdisp,'r-','LineWidth',2)
title('Density difference');
legend('p(x)','p''(x)','f(x)','fh(x)');
xlabel('x');
axis([min(xdisp) max(xdisp) ymin-ymargin ymax+ymargin])
set(gcf,'PaperUnits','centimeters');
set(gcf,'PaperPosition',[0 0 12 9]);
%print('-depsc',sprintf('densities%g',dataset))
print('-dpng',sprintf('densities%g',dataset))

disp(sprintf('L2-distance (true) = %g',L2dist_true));
disp(sprintf('L2-distance (LSDD) = %g',L2dist_LSDD));
